@@ -160,6 +160,31 @@ public:
     out->erase(out->begin() + k, out->end());
   }
 
+  inline static void Assign(std::vector<VAL_T>* array, VAL_T t, size_t n) {
+    array->resize(n);
+    for (size_t i = 0; i < array->size(); ++i) {
+      (*array)[i] = t;
+    }
+  }
+
+  inline static bool CheckAllZero(const std::vector<VAL_T>& array) {
+    for (size_t i = 0; i < array.size(); ++i) {
+      if (array[i] != VAL_T(0)) {
+        return false;
+      }
+    }
+    return true;
+  }
+
+  inline static bool CheckAll(const std::vector<VAL_T>& array, VAL_T t) {
+    for (size_t i = 0; i < array.size(); ++i) {
+      if (array[i] != t) {
+        return false;
+      }
+    }
+    return true;
+  }
+
 };
 
 }  // namespace LightGBM
