@@ -12,6 +12,7 @@ is_py3 = (sys.version_info[0] == 3)
 
 """compatibility between python2 and python3"""
 if is_py3:
+    zip_ = zip
     string_type = str
     numeric_types = (int, float, bool)
     integer_types = (int, )
@@ -24,6 +25,7 @@ if is_py3:
     def decode_string(bytestring):
         return bytestring.decode('utf-8')
 else:
+    from itertools import izip as zip_
     string_type = basestring
     numeric_types = (int, long, float, bool)
     integer_types = (int, long)
